@@ -14,7 +14,7 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
-#' Non-Negative Sparse PCA
+#' Non-Negative and Sparse PCA
 #' 
 #' Performs a constrained principal component analysis,
 #' where non-negativity and/or sparsity constraints are enforced on the principal axes.
@@ -49,92 +49,92 @@ nsprcomp <- function (x, ...) UseMethod("nsprcomp")
 #' @method nsprcomp default
 #' @S3method nsprcomp default
 #' @rdname nsprcomp
-#' @param retx a logical value indicating whether the principal components,
-#'   i.e. \code{x} projected into the principal subspace, should be returned.
-#' @param ncomp the number of principal components (PCs) 
-#'   to be computed. With the default setting, PCs are 
-#'   computed until \code{x} is fully deflated. \code{ncomp} can be specified 
-#'   implicitly if \code{k} is given as a vector.
-#' @param omega a vector with as many entries as there are data samples, to
-#'   perform weighted PCA (analogous to weighted least-squares regression). 
-#'   The default is an equal weighting of all samples.
-#' @param k either a scalar or a vector of length \code{ncomp}, specifying the
+#' @param retx a logical value indicating whether the principal components, i.e.
+#'   \code{x} projected into the principal subspace, should be returned.
+#' @param ncomp the number of principal components (PCs) to be computed. With 
+#'   the default setting, PCs are computed until \code{x} is fully deflated. 
+#'   \code{ncomp} can be specified implicitly if \code{k} is given as a vector.
+#' @param omega a vector with as many entries as there are data samples, to 
+#'   perform weighted PCA (analogous to weighted least-squares regression). The 
+#'   default is an equal weighting of all samples.
+#' @param k either a scalar or a vector of length \code{ncomp}, specifying the 
 #'   upper bounds on the cardinalities of the principal axes (PAs).
 #' @param nneg a logical value indicating whether the loadings should be 
-#'   non-negative, i.e. the PAs should be constrained to the non-negative orthant.
-#' @param center a logical value indicating whether the empirical mean of \code{x}
-#'   should be subtracted. Alternatively, a vector of
-#'   length equal the number of columns of \code{x} can be supplied.
-#'   The value is passed to \code{\link{scale}}.
-#' @param scale. a logical value indicating whether the columns of \code{x} should
-#'   be scaled to have unit variance before the analysis takes
-#'   place. The default is \code{FALSE} for consistency with \code{prcomp}.
-#'   Alternatively, a vector of length
-#'   equal the number of columns of \code{x} can be supplied.  The
+#'   non-negative, i.e. the PAs should be constrained to the non-negative 
+#'   orthant.
+#' @param center a logical value indicating whether the empirical mean of (the 
+#'   columns) of \code{x} should be subtracted. Alternatively, a vector of 
+#'   length equal to the number of columns of \code{x} can be supplied. The 
 #'   value is passed to \code{\link{scale}}.
-#' @param tol a threshold indicating the magnitude below which components
-#'   should be omitted. Components are omitted if their
-#'   standard deviations are less than or equal to \code{tol} times the
-#'   standard deviation of the first component.
-#'   With the default \code{NULL} setting, no components
-#'   are omitted.  With \code{tol = 0} or \code{tol = sqrt(.Machine$double.eps)}, 
-#'   essentially constant components are omitted.
-#' @param nrestart the number of random restarts for computing
-#'   the principal component via expectation-maximization (EM) iterations. The solution 
+#' @param scale. a logical value indicating whether the columns of \code{x} 
+#'   should be scaled to have unit variance before the analysis takes place. The
+#'   default is \code{FALSE} for consistency with \code{prcomp}. Alternatively, 
+#'   a vector of length equal to the number of columns of \code{x} can be 
+#'   supplied. The value is passed to \code{\link{scale}}.
+#' @param tol a threshold indicating the magnitude below which components should
+#'   be omitted. Components are omitted if their standard deviations are less 
+#'   than or equal to \code{tol} times the standard deviation of the first 
+#'   component. With the default \code{NULL} setting, no components are omitted.
+#'   With \code{tol = 0} or \code{tol = sqrt(.Machine$double.eps)}, essentially 
+#'   constant components are omitted.
+#' @param nrestart the number of random restarts for computing the principal 
+#'   component via expectation-maximization (EM) iterations. The solution 
 #'   achieving maximum standard deviation over all random restarts is kept. A 
-#'   value greater than one can help to avoid bad local maxima.
-#' @param em_tol If the relative change
-#'   of the objective is less than \code{em_tol} between iterations, 
-#'   the EM procedure is asssumed to have converged to a local optimum.
-#' @param em_maxiter the maximum number of EM iterations to be performed. The
-#'   EM procedure is terminated if either the \code{em_tol} or the \code{em_maxiter}
-#'   criterion is satisfied.
-#' @param rety a logical value indicating whether the deflated data matrix
-#'   should be returned.
-#' @param verbosity an integer specifying the verbosity level. Greater values
+#'   value greater than one can help to avoid poor local maxima.
+#' @param em_tol If the relative change of the objective is less than 
+#'   \code{em_tol} between iterations, the EM procedure is asssumed to have 
+#'   converged to a local optimum.
+#' @param em_maxiter the maximum number of EM iterations to be performed. The EM
+#'   procedure is terminated if either the \code{em_tol} or the 
+#'   \code{em_maxiter} criterion is satisfied.
+#' @param partial_model \code{NULL} or an object of class \code{nsprcomp}. The 
+#'   computation can be continued from a partial model by providing a 
+#'   \code{nsprcomp} object (either from a previous run of this function or from
+#'   \code{\link{asdev}}) and setting \code{ncomp} to a value greater than the 
+#'   number of components contained in the partial model. See the examples for 
+#'   an illustration.
+#' @param verbosity an integer specifying the verbosity level. Greater values 
 #'   result in more output, the default is to be quiet.
-#' 
-#' @return \code{nsprcomp} returns a list with class \code{(nsprcomp, prcomp)}
-#' containing the following elements:
-#' \item{sdev}{the additional standard deviation explained by each component,
-#'   see \code{\link{asdev}}.}
-#' \item{rotation}{the matrix of non-negative and/or sparse loadings, 
-#'   containing the principal axes as columns.}
-#' \item{x}{the scores matrix \eqn{\mathbf{XW}}{X*W} containing the 
-#'   principal components as columns (after centering and scaling 
-#'   if requested). For the formula method, \code{\link{napredict}} is applied 
-#'   to handle the treatment of values omitted by the \code{na.action}.}
-#' \item{center, scale}{the centering and scaling used, or \code{FALSE}.}
-#' \item{y}{if \code{rety} is \code{TRUE} the deflated data matrix, for which all
-#'   principal axes lie in its null space.}
 #'   
-#' @note The PCA terminology is not consistent across the literature. Given a zero
-#' mean data matrix \eqn{\mathbf{X}}{X} (with observations as rows) and a basis 
-#' \eqn{\mathbf{W}}{W} of the principal subspace,
-#' we define the scores matrix as \eqn{\mathbf{Z}=\mathbf{XW}}{Z=X*W} which 
-#' contains the principal components 
-#' as its columns. The columns of \eqn{\mathbf{W}}{W} are called the principal axes, and the 
-#' elements of \eqn{\mathbf{W}}{W} are called the loadings.
-#' 
-#' Deflating the data matrix accumulates numerical errors over successive
-#' PCs.
+#' @return \code{nsprcomp} returns a list with class \code{(nsprcomp, prcomp)} 
+#'   containing the following elements: \item{sdev}{the additional standard 
+#'   deviation explained by each component, see \code{\link{asdev}}.} 
+#'   \item{rotation}{the matrix of non-negative and/or sparse loadings, 
+#'   containing the principal axes as columns.} \item{x}{the scores matrix 
+#'   \eqn{\mathbf{XW}}{X*W} containing the principal components as columns 
+#'   (after centering and scaling if requested). For the formula method, 
+#'   \code{\link{napredict}} is applied to handle the treatment of values 
+#'   omitted by the \code{na.action}.} \item{center, scale}{the centering and 
+#'   scaling used, or \code{FALSE}} \item{xp}{the deflated data matrix 
+#'   corresponding to \code{x}} \item{q}{an orthonormal basis for the principal 
+#'   subspace}
+#'   
+#' @note The PCA terminology is not consistent across the literature. Given a 
+#'   zero mean data matrix \eqn{\mathbf{X}}{X} (with observations as rows) and a
+#'   basis \eqn{\mathbf{W}}{W} of the principal subspace, we define the scores 
+#'   matrix as \eqn{\mathbf{Z}=\mathbf{XW}}{Z=X*W} which contains the principal 
+#'   components as its columns. The columns of the pseudo-rotation matrix
+#'   \eqn{\mathbf{W}}{W} are called the principal axes, and the elements of
+#'   \eqn{\mathbf{W}}{W} are called the loadings.
+#'   
+#'   Deflating the data matrix accumulates numerical errors over successive PCs.
 #'   
 #' @references Sigg, C. D. and Buhmann, J. M. (2008) Expectation-Maximization 
-#'   for Sparse and Non-Negative PCA. In \emph{Proceedings of the 25th International 
-#'   Conference on Machine Learning} (pp. 960--967).
+#'   for Sparse and Non-Negative PCA. In \emph{Proceedings of the 25th 
+#'   International Conference on Machine Learning} (pp. 960--967).
 #' @references Mackey, L. (2009) Deflation Methods for Sparse PCA. In 
 #'   \emph{Advances in Neural Information Processing Systems} (pp. 1017--1024).
 #'   
 #' @seealso  \code{\link{asdev}},  \code{\link{peav}}, \code{\link{prcomp}}, 
 #'   \code{\link{scale}}
-#' 
+#'   
 #' @example inst/nsprcomp_examples.R
 nsprcomp.default <- function(x, retx = TRUE, ncomp = min(dim(x)), 
                              omega = rep(1, nrow(x)),
                              k = ncol(x), nneg = FALSE, 
                              center = TRUE, scale. = FALSE, tol = NULL,
                              nrestart = 5, em_tol = 1e-3, em_maxiter = 100,
-                             rety = FALSE, 
+                             partial_model = NULL,
                              verbosity = 0, ...) {  
     
     d <- ncol(x); n <- nrow(x)
@@ -153,9 +153,29 @@ nsprcomp.default <- function(x, retx = TRUE, ncomp = min(dim(x)),
     
     sdev <- rep(0, ncomp)  # additional explained standard deviations
     W <- matrix(0, d, ncomp)  # principal axes as columns
-    Q <- matrix(0, d, ncomp)  # orthonormal basis for principal subspace
+    Q <- matrix(0, d, ncomp)  # orthonormal basis for the principal subspace
     Xp <- X  # data matrix, to be deflated if ncomp > 1
-    for (cc in seq(ncomp)) {
+    attr(Xp, "scaled:center") <- NULL
+    attr(Xp, "scaled:scale") <- NULL
+    ccs <- seq(ncomp)
+
+    # start from partially completed model
+    if (!is.null(partial_model)) {
+        if (!("nsprcomp" %in% class(partial_model))) {
+            stop("argument 'partial_model' must have class 'nsprcomp'")
+        }
+
+        pcomp <- length(partial_model$sdev)  # number of components in partial model
+        if (pcomp >= ncomp)
+            stop("'ncomp' must exceed the number of canonical variables in the partial model")
+        sdev[1:pcomp] <- partial_model$sdev
+        W[ , 1:pcomp] <- partial_model$rotation
+        Q[ , 1:pcomp] <- partial_model$q
+        Xp <- partial_model$xp
+        ccs <- seq(pcomp+1, ncomp)
+    }
+
+    for (cc in ccs) {
         obj_opt <- -Inf
         for (rr in seq(nrestart)) {
             res <- empca(Xp, Q[ , seq_len(cc-1), drop=FALSE], omega, k[cc], 
@@ -186,37 +206,47 @@ nsprcomp.default <- function(x, retx = TRUE, ncomp = min(dim(x)),
         W[ ,cc] <- w
         sdev[cc] <- sd(as.vector(Xp%*%w))  # explicit casting to avoid warning in old R versions
         
+        # early stopping at previous component due to standard deviation threshold
+        if (!is.null(tol) && sdev[cc] < tol*sdev[1]) {
+            if (verbosity > 0) {
+                print("component magnitude below threshold, less than 'ncomp' components could be computed")
+            }
+            sdev <- sdev[1:(cc-1)]
+            W <- W[ , 1:(cc-1), drop=FALSE]
+            Q <- Q[ , 1:(cc-1), drop=FALSE]
+            break
+        }
+
+        # deflate data matrix
         if (cc > 1) {
             q <- w - Q[ , 1:(cc-1)]%*%(t(Q[ , 1:(cc-1)])%*%w) 
         } else {
             q <- w
         }
         q <- q/normv(q)
-        Xp <- Xp - Xp%*%q%*%t(q)   
         Q[ , cc] <- q
+        Xp <- Xp - Xp%*%q%*%t(q)
         
-        if (!is.null(tol) && sdev[cc] < tol*sdev[1]) {
-            sdev <- sdev[1:(cc-1)]
-            W <- W[ , 1:(cc-1), drop=FALSE]
-            break
-        } else if (cc < ncomp && all(abs(Xp) < 1e-14)) { 
+        # early stopping at current component due to fully deflated data matrix
+        if (cc < ncomp && all(abs(Xp) < 1e-14)) {
             if (verbosity > 0) {
                 print("data matrix is fully deflated, less than 'ncomp' components could be computed")            
             }
             sdev <- sdev[1:cc]
             W <- W[ , 1:cc, drop=FALSE]
+            Q <- Q[ , 1:cc, drop=FALSE]
             break
         }
     }
     
     dimnames(W) <- list(colnames(X), paste0("PC", seq_len(ncol(W))))
-    r <- list(sdev = sdev, rotation = W,
+    nspc <- list(sdev = sdev, rotation = W,
               center = if(is.null(cen)) FALSE else cen,
-              scale = if(is.null(sc)) FALSE else sc)
-    if (retx) r$x <- X%*%W
-    if (rety) r$y <- Xp
-    class(r) <- c("nsprcomp", "prcomp")
-    return(r)
+              scale = if(is.null(sc)) FALSE else sc,
+              xp = Xp, q = Q)
+    if (retx) nspc$x <- X%*%W
+    class(nspc) <- c("nsprcomp", "prcomp")
+    return(nspc)
 }
 
 empca <- function(Xp, Q, omega, k, nneg, em_tol, em_maxiter, verbosity) {
