@@ -17,4 +17,4 @@ library(MASS)
 library(testthat)
 library(nsprcomp)
 
-test_package("nsprcomp")
\ No newline at end of file
+test_check("nsprcomp")
\ No newline at end of file
